# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity/app_definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from union.internal.identity import enums_pb2 as identity_dot_enums__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1didentity/app_definition.proto\x12\x11\x63loudidl.identity\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x14identity/enums.proto\"\xbb\x06\n\x03\x41pp\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x1b\n\tclient_id\x18\x02 \x01(\tR\x08\x63lientId\x12I\n\x13\x63lient_id_issued_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x63lientIdIssuedAt\x12#\n\rredirect_uris\x18\x04 \x03(\tR\x0credirectUris\x12G\n\x0eresponse_types\x18\x05 \x03(\x0e\x32 .cloudidl.identity.ResponseTypesR\rresponseTypes\x12>\n\x0bgrant_types\x18\x06 \x03(\x0e\x32\x1d.cloudidl.identity.GrantTypesR\ngrantTypes\x12M\n\x10\x61pplication_type\x18\x07 \x01(\x0e\x32\".cloudidl.identity.ApplicationTypeR\x0f\x61pplicationType\x12\x1a\n\x08\x63ontacts\x18\x08 \x03(\tR\x08\x63ontacts\x12\x1f\n\x0b\x63lient_name\x18\t \x01(\tR\nclientName\x12\x19\n\x08logo_uri\x18\n \x01(\tR\x07logoUri\x12\x1d\n\nclient_uri\x18\x0b \x01(\tR\tclientUri\x12\x1d\n\npolicy_uri\x18\x0c \x01(\tR\tpolicyUri\x12\x17\n\x07tos_uri\x18\r \x01(\tR\x06tosUri\x12\x19\n\x08jwks_uri\x18\x0e \x01(\tR\x07jwksUri\x12g\n\x1atoken_endpoint_auth_method\x18\x0f \x01(\x0e\x32*.cloudidl.identity.TokenEndpointAuthMethodR\x17tokenEndpointAuthMethod\x12#\n\rclient_secret\x18\x10 \x01(\tR\x0c\x63lientSecret\x12S\n\x18\x63lient_secret_expires_at\x18\x11 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15\x63lientSecretExpiresAtB\xbf\x01\n\x15\x63om.cloudidl.identityB\x12\x41ppDefinitionProtoH\x02P\x01Z+github.com/unionai/cloud/gen/pb-go/identity\xa2\x02\x03\x43IX\xaa\x02\x11\x43loudidl.Identity\xca\x02\x11\x43loudidl\\Identity\xe2\x02\x1d\x43loudidl\\Identity\\GPBMetadata\xea\x02\x12\x43loudidl::Identityb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'identity.app_definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.cloudidl.identityB\022AppDefinitionProtoH\002P\001Z+github.com/unionai/cloud/gen/pb-go/identity\242\002\003CIX\252\002\021Cloudidl.Identity\312\002\021Cloudidl\\Identity\342\002\035Cloudidl\\Identity\\GPBMetadata\352\002\022Cloudidl::Identity'
  _globals['_APP']._serialized_start=108
  _globals['_APP']._serialized_end=935
# @@protoc_insertion_point(module_scope)
